from xmlbase import HandlerBase, parse, Generic_ElementHandler, NAMESPACE_SEPARATOR
from atom.parse import Atom_EntryHandler, atom_ns

from model import *


APP_NAMESPACE = "http://purl.org/atom/app#"

def app_ns(name):
    return APP_NAMESPACE + NAMESPACE_SEPARATOR + name


class AtomProtocolException(Exception):
    pass


class AtomProtocol_HandlerBase(HandlerBase):
     
    def no_children(self, name, attribute):
        raise AtomProtocolException("%s not allowed here" % name)


## APP-07 7.2.1

class Introspection_RootHandler(AtomProtocol_HandlerBase):

    def child(self, name, attributes):
        if name == app_ns("service"):
            return Introspection_ServiceHandler
        else:
            raise AtomProtocolException("%s not allowed here" % name)


## APP-07 7.2.1 - app:service

class Introspection_ServiceHandler(AtomProtocol_HandlerBase):

    def init(self):
        self.product = Service()

    def child(self, name, attributes):
        if name == app_ns("workspace"):
            return Introspection_WorkspaceHandler
        else: # @@@ extensionElement
            return Generic_ElementHandler

    def end(self, name):
        if len(self.product.workspaces) == 0:
            raise AtomProtocolException("service must have at least one workspace")
        self.parent_handler.product = self.product


## APP-07 7.2.2 - app:workspace

class Introspection_WorkspaceHandler(AtomProtocol_HandlerBase):

    def init(self):
        self.product = Workspace()
        if "title" not in self.attributes:
            raise AtomProtocolException("workspace must have a title")
        self.product.title = self.attributes["title"]
        
    def child(self, name, attributes):
        if name == app_ns("collection"):
            return Introspection_CollectionHandler
        else: # @@@ extensionElement
            return Generic_ElementHandler

    def end(self, name):
        # @@@ APP-07 requires there be at least one collection in a workspace
        self.parent_handler.product.workspaces.append(self.product)


## APP-07 7.2.3 - app:collection

class Introspection_CollectionHandler(AtomProtocol_HandlerBase):

    def init(self):
        self.product = CollectionReference()

        if "title" not in self.attributes:
            raise AtomProtocolException("collection must have 'title' attribute")
        if "href" not in self.attributes:
            raise AtomProtocolException("collection must have 'href' attribute")
            
        for name, value in self.attributes.items():
            if name == "title":
                self.product.title = value
            elif name == "href":
                self.product.href = value
            else:
                raise AtomException("%s not allowed here" % name)

    def child(self, name, attributes):
        if name == app_ns("member-type"):
            return Introspection_MemberTypeHandler
        else: # @@@ extensionElement
            return Generic_ElementHandler

    def end(self, name):
        if self.product.member_type is None:
            raise AtomProtocolException("collection must have a member-type")
        self.parent_handler.product.collections.append(self.product)


## APP-07 7.2.4 - app:member-type

class Introspection_MemberTypeHandler(AtomProtocol_HandlerBase):

    def end(self, name):
        # @@@ should appTypeValue = "entry" | "media" be enforced?
        self.parent_handler.product.member_type = self.char_data
    

# this is version of Atom_RootHandler for the more lax entries that can be 
# POSTed to a collection

class LaxEntry_RootHandler(AtomProtocol_HandlerBase):

    def child(self, name, attributes):
        if name == atom_ns("entry"):
            return LaxEntry_EntryHandler
        else:
            raise AtomProtocolException("document element must be entry")
            

# this is version of Atom_EntryHandler for the more lax entries that can be 
# POSTed to a collection

class LaxEntry_EntryHandler(Atom_EntryHandler):

    def validate(self):
        # unlike superclass, this doesn't have restrictions on id, updated
        if not self.got_title:
            raise AtomException("entry must have title element")


## MAIN FUNCTIONS IN THIS MODULE

def parse_introspection(xml):
     return parse(xml, Introspection_RootHandler)


def parse_entry(xml):
    return parse(xml, LaxEntry_RootHandler)