#!/usr/bin/env python

import traceback
from cStringIO import StringIO
import BaseHTTPServer
import time
import urllib, cgi

from store import UnsupportedMediaTypeError
from app import generate_introspection
from atom import generate_feed


class IntrospectionHandler:

    def HEAD(self, handler):
        handler.send_response(200)
        handler.send_header("Content-type", "application/atomserv+xml")
        handler.end_headers()
        
    def GET(self, handler):
        handler.send_response(200)
        handler.send_header("Content-type", "application/atomserv+xml")
        handler.end_headers()
        generate_introspection(handler.wfile, atom_store.service)
        
    def POST(self, handler):
        handler.send_error(405, "Cannot POST to this URI")

    def PUT(self, handler):
        handler.send_error(405, "Cannot PUT to this URI")

    def DELETE(self, handler):
        handler.send_error(405, "Cannot DELETE this URI")


class CollectionHandler:

    def HEAD(self, handler):
        # get head of collection document

        path, query = urllib.splitquery(handler.path)
        collection = atom_store.get_collection(path)
        
        if collection:
            if query:
                # @@@ cleanup
                try:
                    qsl = cgi.parse_qsl(query, strict_parsing=True)
                    if len(qsl) != 1:
                        raise ValueError
                    if qsl[0][0] != "index":
                        raise ValueError
                    index = int(qsl[0][1])
                except ValueError:
                    handler.send_error(404, "Resource Not Found") # @@@
            else:
                index = 0
            
            feed = collection.make_feed(index)
        
            content = StringIO()
            generate_feed(content, feed)
            handler.send_response(200)
            handler.send_header("Content-type", "application/atom+xml")
            handler.send_header("Content-Length", str(len(content.getvalue())))
            handler.end_headers()
        else:
            handler.send_error(404, "Resource Not Found")
        
    def GET(self, handler):
        # get collection document
        
        path, query = urllib.splitquery(handler.path)
        collection = atom_store.get_collection(path)
        
        if collection:
            if query:
                # @@@ cleanup
                try:
                    qsl = cgi.parse_qsl(query, strict_parsing=True)
                    if len(qsl) != 1:
                        raise ValueError
                    if qsl[0][0] != "index":
                        raise ValueError
                    index = int(qsl[0][1])
                except ValueError:
                    handler.send_error(404, "Resource Not Found") # @@@
            else:
                index = 0
            
            feed = collection.make_feed(index)
        
            content = StringIO()
            generate_feed(content, feed)
            handler.send_response(200)
            handler.send_header("Content-type", "application/atom+xml")
            handler.send_header("Content-Length", str(len(content.getvalue())))
            handler.end_headers()
            handler.wfile.write(content.getvalue())
        else:
            handler.send_error(404, "Collection Not Found")

    def POST(self, handler):
        # add new resource to collection and return 201 with location

        collection = atom_store.get_collection(handler.path)

        # @@@ the following code really belongs in store.py

        if collection:
            suggested_title = handler.headers.get("Title")
            media_type = handler.headers.get("Content-type")
            length = handler.headers.get("Content-Length")

            content = handler.rfile.read(int(length))

            # throw away additional data (from CGIHTTPServer)
            #while select.select([self.rfile._sock], [], [], 0)[0]:
            #    if not self.rfile._sock.recv(1):
            #        break
            
            try:
                new_uri = collection.create_member(content, media_type, suggested_title)
                
                handler.send_response(201)
                handler.send_header("Location", new_uri)
                handler.end_headers()
            except UnsupportedMediaTypeError:
                handler.send_error(415, "Unsupported Media Type")
        else:
            handler.send_error(404, "Collection Not Found")
        
    def PUT(self, handler):
        handler.send_error(405, "Cannot PUT to this URI")

    def DELETE(self, handler):
        handler.send_error(405, "Cannot DELETE this URI")


class MemberHandler:

    def HEAD(self, handler):
        # get head of member resource
        pass # @@@
        
    def GET(self, handler):
        # get member resource
        
        member = atom_store.get_member(handler.path)

        if member:
            handler.send_response(200)
            handler.send_header("Content-type", member.media_type)
            # @@@ length
            handler.end_headers()
            handler.wfile.write(member.get_content())
        else:
            handler.send_error(404, "Resource Not Found")

    def POST(self, handler):
        handler.send_error(405, "Cannot POST to this URI")

    def PUT(self, handler):
        # replace resource, return 200 with resource

        media_type = handler.headers.get("Content-type")
        length = handler.headers.get("Content-Length")
        suggested_title = handler.headers.get("Title")

        content = handler.rfile.read(int(length))

        # throw away additional data (from CGIHTTPServer)
        #while select.select([self.rfile._sock], [], [], 0)[0]:
        #    if not self.rfile._sock.recv(1):
        #        break

        if atom_store.member_exists(handler.path):
            member = atom_store.get_member(handler.path)
        else:
            member = atom_store.create_member(handler.path)
            # @@@ won't be in any collections        
    
        atom_store.replace_member(member, content, media_type, suggested_title)

        handler.send_response(200)
        handler.send_header("Content-type", media_type)
        # @@@ length
        handler.end_headers()
        handler.wfile.write(content)
            
    def DELETE(self, handler):
        # delete resource, return 204
        
        member = atom_store.get_member(handler.path)
        if member:
            atom_store.remove_member(member)
        else:
            handler.send_error(404, "Resource Not Found")
        
        handler.send_response(204)


class AtomRequestHandler(BaseHTTPServer.BaseHTTPRequestHandler):
    
    server_version = "Demokritos/0.2"
    
    def do_HEAD(self):
        try:
            resource_handler = self.dispatch(self.path)
            resource_handler.HEAD(self)
        except:
            traceback.print_exc()
            self.send_error(500, "Server Got Exception")

    def do_GET(self):
        try:
            resource_handler = self.dispatch(self.path)
            resource_handler.GET(self)
        except:
            traceback.print_exc()
            self.send_error(500, "Server Got Exception")
        
    def do_POST(self):
        try:
            resource_handler = self.dispatch(self.path)
            resource_handler.POST(self)
        except:
            traceback.print_exc()
            self.send_error(500, "Server Got Exception")
            
    def do_PUT(self):
        try:
            resource_handler = self.dispatch(self.path)
            resource_handler.PUT(self)
        except:
            traceback.print_exc()
            self.send_error(500, "Server Got Exception")
        
    def do_DELETE(self):
        try:
            resource_handler = self.dispatch(self.path)
            resource_handler.DELETE(self)
        except:
            traceback.print_exc()
            self.send_error(500, "Server Got Exception")
        
    def dispatch(self, path):
        if path == "/introspection":
            return IntrospectionHandler()
        elif atom_store.collection_exists(urllib.splitquery(path)[0]):
            return CollectionHandler()
        else:
            return MemberHandler()


class AtomServer:

    def __init__(self, host="", port=8000):
        self.httpd = BaseHTTPServer.HTTPServer((host, port), AtomRequestHandler)
        
    def serve_forever(self):
        self.httpd.serve_forever()