#!/usr/bin/env python

import httplib, socket, urlparse

import sys
sys.path.append("../lib")
from app import parse_introspection
from atom import parse_atom


SERVER = "localhost:8000"

def request(server, method, path, body=None, headers=None):
    conn = httplib.HTTPConnection(server)
    try:
        print "trying %s %s to %s" % (method, path, server)
        if body is not None and headers is not None:
            conn.request(method, path, body, headers)
        else:
            conn.request(method, path)
        resp = conn.getresponse()
        conn.close()
        print "got %s %s" % (resp.status, resp.reason)
        return resp
    except socket.error:
        print "got socket error"
        return None

## Retrieving an Introspection Document (APP-07 5.1)
    
print "getting introspection document..."

collection_href = None
    
response = request(SERVER, "GET", "/introspection")
if response:
    assert response.status == 200
    doc = response.read()
    service = parse_introspection(doc)
    print "service:"
    for workspace in service.workspaces:
        print "\tworkspace:", workspace.title
        for collection in workspace.collections:
            print "\t\tcollection:", collection.title
            print "\t\t\thref =", collection.href
            print "\t\t\tmember-type =", collection.member_type
    collection_href = service.workspaces[0].collections[0].href

raw_input("press return")

## Listing Collection Memembers (APP-07 5.4)

member_href = None
   
if collection_href:
    print "getting first collection in first workspace..."

    server, path = urlparse.urlparse(collection_href)[1:3]
     
    response = request(server, "GET", path)
    if response:
        assert response.status == 200
        doc = response.read()
        collection = parse_atom(doc)        
        print "collection:"
        for entry in collection.entries:
            print "\tmember:", entry.title.value
            print "\t\tupdated:", entry.updated.value
            for link in entry.links:
                print "\t\tlink %s: %s" % (link.rel, link.href)
            print "\t\tcontent src = %s" % entry.content.src

        member_href = collection.entries[0].links[0].href

raw_input("press return")

##

if member_href:
    print "getting first member in that collection..."
    
    server, path = urlparse.urlparse(member_href)[1:3]
     
    response = request(server, "GET", path)
    if response and response.status == 200:
        doc = response.read()
        print doc

raw_input("press return")

## Creating a Resource (APP-07 5.2)

location = None
        
if collection_href:
    print "POSTing to collection..."
    
    server, path = urlparse.urlparse(collection_href)[1:3]
     
    body = "Hello world!"
    headers = {"Content-type": "text/plain", "Title": "My Title"}
    response = request(server, "POST", path, body, headers)
    if response:
        assert response.status == 201
        location = response.getheader("Location")
        print "got location:", location
        
raw_input("press return")

## Listing Collection Memembers (APP-07 5.4)

if collection_href:
    print "getting collection we just posted to..."

    server, path = urlparse.urlparse(collection_href)[1:3]
     
    response = request(server, "GET", path)
    if response:
        assert response.status == 200
        doc = response.read()
        collection = parse_atom(doc)        
        print "collection:"
        for entry in collection.entries:
            print "\tmember:", entry.title.value
            print "\t\tupdated:", entry.updated.value
            for link in entry.links:
                print "\t\tlink %s: %s" % (link.rel, link.href)
            print "\t\tcontent src = %s" % entry.content.src

        member_href = collection.entries[0].links[0].href

raw_input("press return")

## Retrieving a Resource (APP-07 5.3.1)

if location:
    print "GETting new resource..."
    
    server, path = urlparse.urlparse(location)[1:3]
     
    response = request(server, "GET", path)
    if response:
        assert response.status == 200
        doc = response.read()
        print doc
        assert doc == "Hello world!"

raw_input("press return")

## Updating a Resource (APP-07 5.3.2)

if collection_href:
    print "PUTing new resource..."
    
    server, path = urlparse.urlparse(location)[1:3]
     
    body = "Goodbye world!"
    headers = {"Content-type": "text/plain"}
    response = request(server, "PUT", path, body, headers)
    if response:
        assert response.status == 200
        doc = response.read()
        print doc
        assert doc == "Goodbye world!"

raw_input("press return")    

##

if location:
    print "GETting newer resource..."
    
    server, path = urlparse.urlparse(location)[1:3]
     
    response = request(server, "GET", path)
    if response:
        assert response.status == 200
        doc = response.read()
        print doc
        assert doc == "Goodbye world!"

raw_input("press return")

## Deleting a Resource (APP-07 5.3.3)

if location:
    print "DELETEing resource..."
    
    server, path = urlparse.urlparse(location)[1:3]
     
    response = request(server, "DELETE", path)
    if response:
        assert response.status == 204

raw_input("press return")

##

if location:
    print "GETting deleted resource..."
    
    server, path = urlparse.urlparse(location)[1:3]
     
    response = request(server, "GET", path)
    if response:
        assert response.status == 404

raw_input("press return")

## Listing Collection Memembers (APP-07 5.4)

if collection_href:
    print "getting collection we just deleted from..."

    server, path = urlparse.urlparse(collection_href)[1:3]
     
    response = request(server, "GET", path)
    if response:
        assert response.status == 200
        doc = response.read()
        collection = parse_atom(doc)        
        print "collection:"
        for entry in collection.entries:
            print "\tmember:", entry.title.value
            print "\t\tupdated:", entry.updated.value
            for link in entry.links:
                print "\t\tlink %s: %s" % (link.rel, link.href)
            print "\t\tcontent src = %s" % entry.content.src

raw_input("press return")

### atom collection tests

##

collection_href = None

response = request(SERVER, "GET", "/introspection")
if response:
    assert response.status == 200
    doc = response.read()
    service = parse_introspection(doc)
    collection_href = service.workspaces[0].collections[1].href

if collection_href:
    print "getting second collection in first workspace..."

    server, path = urlparse.urlparse(collection_href)[1:3]
     
    response = request(server, "GET", path)
    if response:
        assert response.status == 200
        doc = response.read()
        collection = parse_atom(doc)        
        print "collection:"
        for entry in collection.entries:
            print "\tmember:", entry.title.value
            print "\t\tupdated:", entry.updated.value
            for link in entry.links:
                print "\t\tlink %s: %s" % (link.rel, link.href)
            print "\t\tcontent src = %s" % entry.content.src

raw_input("press return")

##

if collection_href:
    print "POSTing non-atom to entry collection..."
    
    server, path = urlparse.urlparse(collection_href)[1:3]
     
    body = "Hello world!"
    headers = {"Content-type": "text/plain", "Title": "My Title"}
    response = request(server, "POST", path, body, headers)
    if response:
        assert response.status == 415

raw_input("press return")

## Creating resources with POST (APP-07 8.1)

location = None
        
if collection_href:
    print "POSTing atom to entry collection..."
    
    server, path = urlparse.urlparse(collection_href)[1:3]

    # from APP-07 8.1.1 (@@@ externalise)
    body = """
    <entry xmlns="http://www.w3.org/2005/Atom">
    <title>Atom-Powered Robots Run Amok</title>
    <link href="http://example.org/2003/12/13/atom03"/>
    <id>urn:uuid:1225c695-cfb8-4ebb-aaaa-80da344efa6a</id>
    <updated>2003-12-13T18:30:02Z</updated>
    <summary>Some text.</summary>
    </entry>"""
    
    headers = {"Content-type": "application/atom+xml"}
    response = request(server, "POST", path, body, headers)
    if response:
        assert response.status == 201
        location = response.getheader("Location")
        print "got location:", location

raw_input("press return")

##

if location:
    print "GETting new resource..."
    
    server, path = urlparse.urlparse(location)[1:3]
     
    response = request(server, "GET", path)
    if response:
        assert response.status == 200
        doc = response.read()
        print doc

raw_input("press return")

## Listing Collection Memembers (APP-07 5.4)

if collection_href:
    print "getting collection again"

    server, path = urlparse.urlparse(collection_href)[1:3]
     
    response = request(server, "GET", path)
    if response:
        assert response.status == 200
        doc = response.read()
        collection = parse_atom(doc)        
        print "collection:"
        for entry in collection.entries:
            print "\tmember:", entry.title.value
            print "\t\tupdated:", entry.updated.value
            for link in entry.links:
                print "\t\tlink %s: %s" % (link.rel, link.href)

        assert collection.entries[0].title.value == "Atom-Powered Robots Run Amok"
        assert collection.entries[0].links[0].rel == "edit"
